searchData={"items":[{"type":"module","title":"Data.Constructor","doc":"","ref":"Data.Constructor.html"},{"type":"function","title":"Data.Constructor.struct/3","doc":"Define and run a smart constructor on a Key-Value input, returning either\nwell-defined `structs` or descriptive errors. The motto: parse, don't validate!\n\nGiven a list of `Data.Parser.KV.field_spec/2`s, a `module`, and an input\n`map` or `Keyword`, create and run a parser which will either parse\nsuccessfully and return an `{:ok, %__MODULE__{}}` struct, or fail and return\nan `{:error, Error.t}` with details about the parsing failure.","ref":"Data.Constructor.html#struct/3"},{"type":"function","title":"Examples - Data.Constructor.struct/3","doc":"iex> defmodule SensorReading do\n    ...>   defstruct [:sensor_id, :microfrobs, :datetime]\n    ...>   def new(input) do\n    ...>     Data.Constructor.struct([\n    ...>      {:sensor_id, Data.Parser.BuiltIn.string()},\n    ...>      {:microfrobs, Data.Parser.BuiltIn.integer()},\n    ...>      {:datetime, Data.Parser.BuiltIn.datetime()}],\n    ...>     __MODULE__,\n    ...>     input)\n    ...>   end\n    ...> end\n    ...>\n    ...> {:ok, reading} = SensorReading.new(sensor_id: \"1234-1234-1234\",\n    ...>                                    microfrobs: 23,\n    ...>                                    datetime: ~U[2018-12-20 12:00:00Z])\n    ...>\n    ...> ~U[2018-12-20 12:00:00Z] = reading.datetime\n    ...> 23 = reading.microfrobs\n    ...> \"1234-1234-1234\" = reading.sensor_id\n    ...> {:error, e} = SensorReading.new(%{\"sensor_id\" => nil,\n    ...>                                   \"microfrobs\" => 23,\n    ...>                                   \"datetime\" => \"2018-12-20 12:00:00Z\"})\n    ...> :failed_to_parse_field = Error.reason(e)\n    ...>  %{field: :sensor_id, input: %{\"datetime\" => \"2018-12-20 12:00:00Z\",\n    ...>                                \"microfrobs\" => 23,\n    ...>                                \"sensor_id\" => nil}} = Error.details(e)\n    ...> {:just, inner_error} = Error.caused_by(e)\n    ...> Error.reason(inner_error)\n    :not_a_string","ref":"Data.Constructor.html#struct/3-examples"},{"type":"function","title":"Data.Constructor.update/3","doc":"Define a smart update function based on a list of field specifications,\nthe struct type to be updated, and a `Keyword` or `map` of input params.\n\n\nGiven a list of `Data.Parser.KV.field_spec/2`s, a `module` (which defines a\nstruct), and an input `map` or `Keyword`, create an `{:ok, &fun/1}` tuple,\nor fail and return an `{:error, Error.t}` with details about the parsing\nfailure.\n\nThe `&fun/1` in the ok-tuple can be applied to any struct as defined by\n`module`, and will update it with the fields provided in the constructor\ninput.\n\nThe crucial advantage here is that the input parameters are validated\naccording to the provided `Data.Parser.KV.field_spec/2`s, so that using the\nsame list of field_specs for `new/3` and `update/3` will result in\ncorrect-by-construction data both from construction and after updates.\n\nAdditionally, if any of the field_specs define a `default:` value, that value\nwill be explicitly allowed in updates for that field, along with the\nspecified type.","ref":"Data.Constructor.html#update/3"},{"type":"function","title":"Examples - Data.Constructor.update/3","doc":"iex> defmodule ReadingWComment do\n    ...>   defstruct [:sensor_id, :microfrobs, :datetime, :comments]\n    ...>\n    ...>   defp fields, do: [\n    ...>      {:sensor_id, Data.Parser.BuiltIn.string()},\n    ...>      {:microfrobs, Data.Parser.BuiltIn.integer()},\n    ...>      {:datetime, Data.Parser.BuiltIn.datetime()},\n    ...>      {:comments, Data.Parser.BuiltIn.string(), default: nil}]\n    ...>\n    ...>   def new(input) do\n    ...>     Data.Constructor.struct(fields(), __MODULE__, input)\n    ...>   end\n    ...>\n    ...>   def update(sensor_reading, input) do\n    ...>     case Data.Constructor.update(fields(), __MODULE__, input) do\n    ...>      {:ok, update_fun} -> update_fun.(sensor_reading)\n    ...>      {:error, e} -> {:error, e}\n    ...>     end\n    ...>   end\n    ...> end\n    ...>\n    ...> {:ok, reading} = ReadingWComment.new(sensor_id: \"1234-1234-1234\",\n    ...>                                      microfrobs: 23,\n    ...>                                      datetime: ~U[2018-12-20 12:00:00Z],\n    ...>                                      comments: \"delete me later\")\n    ...> \"delete me later\" = reading.comments\n    ...>\n    ...>\n    ...> {:ok, reading2} = ReadingWComment.update(reading,\n    ...>                                          microfrobs: 25,\n    ...>                                          datetime: ~U[2018-12-20 13:00:00Z])\n    ...> ~U[2018-12-20 13:00:00Z] = reading2.datetime\n    ...> 25 = reading2.microfrobs\n    ...>\n    ...>\n    ...> {:ok, reading3} = ReadingWComment.update(reading2,\n    ...>                                          comments: nil)\n    ...> nil = reading3.comments\n    ...>\n    ...>\n    ...> {:error, e} = ReadingWComment.update(reading3,\n    ...>                                      microfrobs: [1,2,3])\n    ...> :invalid_parameter = Error.reason(e)\n    ...> %{key: :microfrobs, value: [1,2,3]} = Error.details(e)\n    ...>\n    ...>\n    ...> {:error, e} = ReadingWComment.update(%{\"my\" => \"special\", \"map\" => \"type\"},\n    ...>                                      microfrobs: 25,\n    ...>                                      datetime: ~U[2018-12-20 13:00:00Z])\n    ...> :struct_type_mismatch = Error.reason(e)\n    ...> Error.details(e)\n    %{expecting: Data.ConstructorTest.ReadingWComment, got: %{\"map\" => \"type\", \"my\" => \"special\"}}","ref":"Data.Constructor.html#update/3-examples"},{"type":"module","title":"Data.Enum","doc":"Functions which extend built-in Enum functionality.","ref":"Data.Enum.html"},{"type":"function","title":"Data.Enum.atomize_keys!/2","doc":"Takes a key-value-type Enum (`map`, `MapSet`, associative list) with binary\nkeys, and returns a `Keyword`, where all keys have been converted to atoms.\n\nThe optional paramter `:safe`, which defaults to `true`, determines whether\nnew atoms should be generated dynamically on-the-fly.\n\nIf `:safe` is `true` and an unknown atom would have to be created, this\nfunction throws an `ArgumentError`.  If `:safe` is `false`, then a new atom\nis created.\n\nGenerating new atoms on-the-fly poses a security risk in prodution systems\nand should be avoided.","ref":"Data.Enum.html#atomize_keys!/2"},{"type":"function","title":"Examples - Data.Enum.atomize_keys!/2","doc":"# note: the atom :hello does not exist in the VM\n    iex> Data.Enum.atomize_keys!(%{\"hello\" => \"world\"})\n    ** (ArgumentError) argument error\n\n    # note: now, the atom does exist!\n    iex> :present; Data.Enum.atomize_keys!(%{\"present\" => \"yes!\"})\n    [present: \"yes!\"]\n\n    # now, we'll generate a new atom\n    iex> Data.Enum.atomize_keys!(%{\"something_new\" => \"DANGER ZONE\"},\n    ...> safe: false)\n    [something_new: \"DANGER ZONE\"]","ref":"Data.Enum.html#atomize_keys!/2-examples"},{"type":"module","title":"Data.Parser","doc":"Higher-order functions to create and modify parsers.","ref":"Data.Parser.html"},{"type":"function","title":"Data.Parser.kv/1","doc":"See  Data.Parser.KV.new/1 .","ref":"Data.Parser.html#kv/1"},{"type":"function","title":"Data.Parser.list/1","doc":"Takes a parser `p` and creates a parser that will successfully parse lists of values that all satisfy `p`.\n\nSpecifically, the input:\n\n1) Must be a list\n\n2) `p` must parse successfully all elements in the input\n\nIf this is the case, the output will be `{:ok, list_of_parsed_values}`.\n\nIf not all values can be parsed with `p`, the result will be the original\nparse error, enriched with the field `:failed_element` in the error details.\n\nIf the input is not a list, the domain error `:not_a_list` will be returned.","ref":"Data.Parser.html#list/1"},{"type":"function","title":"Examples - Data.Parser.list/1","doc":"iex> Data.Parser.list(Data.Parser.BuiltIn.integer()).([])\n    {:ok, []}\n\n    iex> Data.Parser.list(Data.Parser.BuiltIn.integer()).([1,2,3])\n    {:ok, [1, 2, 3]}\n\n    iex> {:error, e} = Data.Parser.list(Data.Parser.BuiltIn.integer()).(%{a: :b})\n    ...> Error.reason(e)\n    :not_a_list\n\n    iex> {:error, e} = Data.Parser.list(Data.Parser.BuiltIn.integer()).([1, :b, 3])\n    ...> Error.reason(e)\n    :not_an_integer\n    ...> Error.details(e)\n    %{failed_element: :b}","ref":"Data.Parser.html#list/1-examples"},{"type":"function","title":"Data.Parser.map/2","doc":"Takes a key parser and a value parser and creates a parser that will\nparse maps of keys and values.\n\nSpecifically, the input:\n\n1) Must be a map\n\n2) All keys of the input map must be parsed correctly by the key parser\n\n3) All values of the input map must be parsed correctly by the value parser\n\nIf this is the case, the output will be `{:ok, map_of_parsed_keys_and_values}`.\n\nIf not all keys can be parsed with the key parser, the result will be the\noriginal parse error, enriched with the field `:failed_key` in the error details.\n\nIf not all values can be parsed with the value parser, the result will be the\noriginal parse error, enriched with the field `:failed_value` in the error details.\n\nIf the input is not a map, the domain error `:not_a_map` will be returned.","ref":"Data.Parser.html#map/2"},{"type":"function","title":"Examples - Data.Parser.map/2","doc":"iex> Data.Parser.map(Data.Parser.BuiltIn.string(), Data.Parser.BuiltIn.integer()).(%{})\n    {:ok, %{}}\n\n    iex> Data.Parser.map(Data.Parser.BuiltIn.string(), Data.Parser.BuiltIn.integer()).(%{\"a\" => 1, \"b\" => 2})\n    {:ok, %{\"a\" => 1, \"b\" => 2}}\n\n    iex> {:error, e} = Data.Parser.map(Data.Parser.BuiltIn.string(), Data.Parser.BuiltIn.integer()).([])\n    ...> Error.reason(e)\n    :not_a_map\n\n    iex> {:error, e} = Data.Parser.map(Data.Parser.BuiltIn.string(), Data.Parser.BuiltIn.integer()).(%{:a => 1})\n    ...> Error.reason(e)\n    :failed_to_parse_key\n    ...> Error.details(e)\n    %{key: :a}\n\n    iex> {:error, e} = Data.Parser.map(Data.Parser.BuiltIn.string(), Data.Parser.BuiltIn.integer()).(%{\"a\" => \"not_int\"})\n    ...> Error.reason(e)\n    :failed_to_parse_value\n    ...> Error.details(e)\n    %{key: \"a\", value: \"not_int\"}","ref":"Data.Parser.html#map/2-examples"},{"type":"function","title":"Data.Parser.maybe/1","doc":"Takes a parser and transforms it so that it works ~c\"inside\" `Maybe.t` values.\n\nIf the original parser works on `String.t()`, the new one will work on\n`Maybe.t(String.t())`.\n\nSuccessful parses on `just()` values return `{:ok, {:just, result_of_parse}}`.\nUnsuccessful parses on `just()` values reutrn `{:error, parse_error}`.\n\nThe parser will successfully return `{:ok, :nothing}` when applied to `:nothing`.","ref":"Data.Parser.html#maybe/1"},{"type":"function","title":"Examples - Data.Parser.maybe/1","doc":"iex(2)> Data.Parser.maybe(\n    ...> Data.Parser.predicate( &is_binary/1, :invalid)).({:just, \"good\"})\n    {:ok, {:just, \"good\"}}\n\n    iex> Data.Parser.maybe(\n    ...> Data.Parser.predicate( &is_binary/1, :invalid)).({:just, ~c\"bad\"})\n    {:error, :invalid}\n\n    iex> Data.Parser.maybe(\n    ...> Data.Parser.predicate( &is_binary/1, :invalid)).(:nothing)\n    {:ok, :nothing}","ref":"Data.Parser.html#maybe/1-examples"},{"type":"function","title":"Data.Parser.nonempty_list/1","doc":"Creates a parser that behaves exactly the same as the `list/1` parser, except\nthat it will return the domain error `:empty_list` if applied to an empty list.","ref":"Data.Parser.html#nonempty_list/1"},{"type":"function","title":"Examples - Data.Parser.nonempty_list/1","doc":"iex> Data.Parser.nonempty_list(Data.Parser.BuiltIn.integer()).([1, 2, 3])\n    {:ok, [1, 2, 3]}\n\n    iex> {:error, e} = Data.Parser.nonempty_list(Data.Parser.BuiltIn.integer()).([1, :b, 3])\n    ...> Error.reason(e)\n    :not_an_integer\n    ...> Error.details(e)\n     %{failed_element: :b}\n\n\n    iex> {:error, e} = Data.Parser.nonempty_list(Data.Parser.BuiltIn.integer()).([])\n    ...> Error.reason(e)\n    :empty_list","ref":"Data.Parser.html#nonempty_list/1-examples"},{"type":"function","title":"Data.Parser.one_of/2","doc":"Takes a list of values, `elements`, and returns a parser that returns\nsuccessfully if its input is present in `elements`.\n\nIf the input is not a member of `elements` and `default` is a value, the\nparser fails with `{:error, default}`. If `default` is a unary function, the\nparser fails with `{:error, default.(input)}`.","ref":"Data.Parser.html#one_of/2"},{"type":"function","title":"Examples - Data.Parser.one_of/2","doc":"iex> Data.Parser.one_of([:he, :ne, :ar, :kr, :xe, :rn], \"not a noble gas\").(:he)\n    {:ok, :he}\n\n    iex> Data.Parser.one_of([:he, :ne, :ar, :kr, :xe, :rn], \"not a noble gas\").(:n)\n    {:error, \"not a noble gas\"}\n\n    iex> Data.Parser.one_of([:he, :ne, :ar, :kr, :xe, :rn],\n    ...> fn x -> \"not a noble gas: #{inspect x}\" end).(:o)\n    {:error, \"not a noble gas: :o\"}","ref":"Data.Parser.html#one_of/2-examples"},{"type":"function","title":"Data.Parser.predicate/1","doc":"Takes a boolean function `p` (the predicate), and returns a parser\nthat parses successfully those values for which `p` is `true`.\n\nIf the predicate returns `false` the parser will return a domain `Error`\nwith the input value and the predicate functions listed in the error details.","ref":"Data.Parser.html#predicate/1"},{"type":"function","title":"Examples - Data.Parser.predicate/1","doc":"iex> {:error, e} = Data.Parser.predicate(&is_binary/1).(~c\"charlists are not ok\")\n    ...> e.reason\n    :predicate_not_satisfied\n    ...> e.details\n    %{input: ~c\"charlists are not ok\", predicate: &is_binary/1}\n\n    iex> Data.Parser.predicate(&is_binary/1).(\"this is fine\")\n    {:ok, \"this is fine\"}\n\n    iex> Data.Parser.predicate(&(&1<10)).(5)\n    {:ok, 5}\n\n    iex> {:error, e} = Data.Parser.predicate(&(&1<10)).(55)\n    ...> e.details.input\n    55","ref":"Data.Parser.html#predicate/1-examples"},{"type":"function","title":"Data.Parser.predicate/2","doc":"Takes a boolean function `p` (the predicate) and a default value, and returns\na parser that parses successfully those values for which `p` is `true`.\n\nIf the predicate function applied to the input returns `true`, the parser\nwraps the input in an `{:ok, input}` tuple.\n\nIf the predicate function returns `false`, and `default` is a value, the\nparser returns `{:error, default}`\n\nIf the predicate returns `false` and `default` is a unary function, the\nparser returns `{:error, default.(the_failed_input)}`.","ref":"Data.Parser.html#predicate/2"},{"type":"function","title":"Examples - Data.Parser.predicate/2","doc":"iex> Data.Parser.predicate(&is_binary/1, \"invalid string\").(~c\"charlists are not ok\")\n    {:error, \"invalid string\"}\n\n    iex> Data.Parser.predicate(&is_binary/1, \"invalid string\").(<<0::1, 1::2>>)\n    {:error, \"invalid string\"}\n\n    iex> Data.Parser.predicate(&is_binary/1, \"invalid string\").(\"this is fine\")\n    {:ok, \"this is fine\"}\n\n    iex> Data.Parser.predicate(&is_binary/1, fn x -> \"the bad value is: #{inspect x}\" end).(12345)\n    {:error, \"the bad value is: 12345\"}","ref":"Data.Parser.html#predicate/2-examples"},{"type":"function","title":"Data.Parser.set/1","doc":"Takes a parser `p` and creates a parser that will successfully parse sets of\nvalues that all satisfy `p`.\n\nSpecifically, the input:\n\n1) must be a `MapSet`\n\n2) all elements of the input set must be parsed correctly by `p`\n\nIf this is the case, the output will be `{:ok, set_of_parsed_values}`.\n\nIf not all values can be parsed with `p`, the result will be the original parse\nerror, enriched with the field `:failed_element` in the error details.\n\nIf the input is not a `MapSet`, the domain error `:not_a_set` will be returned.","ref":"Data.Parser.html#set/1"},{"type":"function","title":"Examples - Data.Parser.set/1","doc":"iex> {:ok, s} = Data.Parser.set(Data.Parser.BuiltIn.integer()).(MapSet.new())\n    ...> s\n    MapSet.new([])\n\n    iex> {:ok, s} = Data.Parser.set(Data.Parser.BuiltIn.integer()).(MapSet.new([1,2,3]))\n    ...> s\n    MapSet.new([1, 2, 3])\n\n    iex> {:error, e} = Data.Parser.set(Data.Parser.BuiltIn.integer()).(%{a: :b})\n    ...> Error.reason(e)\n    :not_a_set\n\n    iex> {:error, e} = Data.Parser.set(Data.Parser.BuiltIn.integer()).(MapSet.new([1, :b, 3]))\n    ...> Error.reason(e)\n    :not_an_integer\n    ...> Error.details(e)\n    %{failed_element: :b}","ref":"Data.Parser.html#set/1-examples"},{"type":"type","title":"Data.Parser.t/2","doc":"A parser is a function that takes any value as input and produces a `Result.t`.\n\nMore specifically, a `parser(a,b)` is a fuction that takes any input and\nreturns `{:ok, a}` on a successful parse or `{:error, b}` if parsing failed.","ref":"Data.Parser.html#t:t/2"},{"type":"function","title":"Data.Parser.union/1","doc":"Takes a list of parsers and creates a parser that returns the first\nsuccessful parse result, or an error listing the parsers and the failed\ninput.","ref":"Data.Parser.html#union/1"},{"type":"function","title":"Examples - Data.Parser.union/1","doc":"iex> Data.Parser.union(\n    ...> [Data.Parser.BuiltIn.integer(),\n    ...>  Data.Parser.BuiltIn.boolean()]).(true)\n    {:ok, true}\n\n    iex> Data.Parser.union(\n    ...> [Data.Parser.BuiltIn.integer(),\n    ...>  Data.Parser.BuiltIn.boolean()]).(1)\n    {:ok, 1}\n\n    iex> {:error, e} = Data.Parser.union(\n    ...>   [Data.Parser.BuiltIn.integer(),\n    ...>    Data.Parser.BuiltIn.boolean()]).(:atom)\n    ...> Error.reason(e)\n    :no_parser_applies\n    ...> Error.details(e).input\n    :atom","ref":"Data.Parser.html#union/1-examples"},{"type":"module","title":"Data.Parser.BuiltIn","doc":"Parsers for built-in Elixir data types.","ref":"Data.Parser.BuiltIn.html"},{"type":"function","title":"Data.Parser.BuiltIn.atom/0","doc":"Creates a parser that successfully parses `atom`s, and returns the\ndomain error `:not_an_atom` for all other inputs.","ref":"Data.Parser.BuiltIn.html#atom/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.atom/0","doc":"iex> Data.Parser.BuiltIn.atom().(:atom)\n    {:ok, :atom}\n\n    iex> Data.Parser.BuiltIn.atom().(:other_atom)\n    {:ok, :other_atom}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.atom().(1.0)\n    ...> Error.reason(e)\n    :not_an_atom\n\n\n    iex> {:error, e} = Data.Parser.BuiltIn.atom().([\"truth\", \"or\", \"dare\"])\n    ...> Error.reason(e)\n    :not_an_atom","ref":"Data.Parser.BuiltIn.html#atom/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.boolean/0","doc":"Creates a parser that successfully parses `boolean`s, and returns the\ndomain error `:not_a_boolean` for all other inputs.","ref":"Data.Parser.BuiltIn.html#boolean/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.boolean/0","doc":"iex> Data.Parser.BuiltIn.boolean().(true)\n    {:ok, true}\n\n    iex> Data.Parser.BuiltIn.boolean().(false)\n    {:ok, false}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.boolean().(1.0)\n    ...> Error.reason(e)\n    :not_a_boolean\n\n\n    iex> {:error, e} = Data.Parser.BuiltIn.boolean().([:truth, :or, :dare])\n    ...> Error.reason(e)\n    :not_a_boolean","ref":"Data.Parser.BuiltIn.html#boolean/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.date/0","doc":"Creates a parser that successfully parses `Date.t`s or `String.t` that\nrepresent legitimate `Date.t`s.\n\nReturns a domain error representing the parse failure if\nthe string input cannot be parsed, and the domain error `:not_a_date`\nfor all other inputs.","ref":"Data.Parser.BuiltIn.html#date/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.date/0","doc":"iex> {:ok, d} = Data.Parser.BuiltIn.date().(~D[1999-12-31])\n    ...> d\n    ~D[1999-12-31]\n\n    iex> {:ok, d} = Data.Parser.BuiltIn.date().(\"1999-12-31\")\n    ...> d\n    ~D[1999-12-31]\n\n    iex> {:error, e} = Data.Parser.BuiltIn.date().(\"19991232\")\n    ...> Error.reason(e)\n    :invalid_format\n\n    iex> {:error, e} = Data.Parser.BuiltIn.date().(\"1999-12-32\")\n    ...> Error.reason(e)\n    :invalid_date\n\n    iex> {:error, e} = Data.Parser.BuiltIn.date().(123456789)\n    ...> Error.reason(e)\n    :not_a_date","ref":"Data.Parser.BuiltIn.html#date/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.datetime/0","doc":"Creates a parser that successfully parses `DateTime.t`s or `String.t` that\nrepresent legitimate `DateTime.t`s.\n\nReturns a domain error representing the parse failure if the string input\ncannot be parsed, and the domain error `:not_a_datetime` for all other inputs.","ref":"Data.Parser.BuiltIn.html#datetime/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.datetime/0","doc":"iex> Data.Parser.BuiltIn.datetime().(~U[1999-12-31 23:59:59Z])\n    {:ok, ~U[1999-12-31 23:59:59Z]}\n\n    iex> Data.Parser.BuiltIn.datetime().(\"1999-12-31 23:59:59Z\")\n    {:ok, ~U[1999-12-31 23:59:59Z]}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.datetime().(\"1999-12-32 23:59:59Z\")\n    ...> Error.reason(e)\n    :invalid_date\n\n    iex> {:error, e} = Data.Parser.BuiltIn.datetime().(\"1999-12-31 23:59:99Z\")\n    ...> Error.reason(e)\n    :invalid_time\n\n    iex> {:error, e} = Data.Parser.BuiltIn.datetime().(\"1999-12-31 23:59:59\")\n    ...> Error.reason(e)\n    :missing_offset\n\n    iex> {:error, e} = Data.Parser.BuiltIn.datetime().(123456789)\n    ...> Error.reason(e)\n    :not_a_datetime","ref":"Data.Parser.BuiltIn.html#datetime/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.float/0","doc":"Creates a parser that successfully parses `float`s, and returns the\ndomain error `:not_a_float` for all other inputs.","ref":"Data.Parser.BuiltIn.html#float/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.float/0","doc":"iex> Data.Parser.BuiltIn.float().(1.0)\n    {:ok, 1.0}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.float().(1)\n    ...> Error.reason(e)\n    :not_a_float\n\n    iex> {:error, e} = Data.Parser.BuiltIn.float().(:hi)\n    ...> Error.reason(e)\n    :not_a_float","ref":"Data.Parser.BuiltIn.html#float/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.integer/0","doc":"Creates a parser that successfully parses `integer`s, and returns the\ndomain error `:not_an_integer` for all other inputs.","ref":"Data.Parser.BuiltIn.html#integer/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.integer/0","doc":"iex> Data.Parser.BuiltIn.integer().(1)\n    {:ok, 1}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.integer().(1.0)\n    ...> Error.reason(e)\n    :not_an_integer\n\n    iex> {:error, e} = Data.Parser.BuiltIn.integer().(:hi)\n    ...> Error.reason(e)\n    :not_an_integer","ref":"Data.Parser.BuiltIn.html#integer/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.naive_datetime/0","doc":"Creates a parser that successfully parses `NaiveDateTime.t`s or `String.t` that\nrepresent legitimate `NaiveDateTime.t`s.\n\nReturns a domain error representing the parse failure if the string input\ncannot be parsed, and the domain error `:not_a_naive_datetime` for all other\ninputs.","ref":"Data.Parser.BuiltIn.html#naive_datetime/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.naive_datetime/0","doc":"iex> Data.Parser.BuiltIn.naive_datetime.(~N[1999-12-31 23:59:59])\n    {:ok, ~N[1999-12-31 23:59:59]}\n\n    iex> Data.Parser.BuiltIn.naive_datetime.(\"1999-12-31 23:59:59\")\n    {:ok, ~N[1999-12-31 23:59:59]}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.naive_datetime.(\"1999-12-32 23:59:59\")\n    ...> Error.reason(e)\n    :invalid_date\n\n    iex> {:error, e} = Data.Parser.BuiltIn.naive_datetime.(\"1999-12-31 23:59:99\")\n    ...> Error.reason(e)\n    :invalid_time\n\n    iex> {:error, e} = Data.Parser.BuiltIn.naive_datetime.(123456789)\n    ...> Error.reason(e)\n    :not_a_naive_datetime","ref":"Data.Parser.BuiltIn.html#naive_datetime/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.null/0","doc":"Creates a parser that successfully parses only `nil`. Any other\nvalues, including `false` and the atom `:nothing` cause the\nparser to fail with a domain error of :not_nil","ref":"Data.Parser.BuiltIn.html#null/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.null/0","doc":"iex> Data.Parser.BuiltIn.null().(nil)\n    {:ok, nil}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.null().(1.0)\n    ...> Error.reason(e)\n    :not_nil\n    \n    iex> {:error, e} = Data.Parser.BuiltIn.null().(false)\n    ...> Error.reason(e)\n    :not_nil","ref":"Data.Parser.BuiltIn.html#null/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.string/0","doc":"Creates a parser that succesfully parses `String.t`s (a.k.a binaries), and\nreturns the domain error `:not_a_string` for all other inputs.","ref":"Data.Parser.BuiltIn.html#string/0"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.string/0","doc":"iex> Data.Parser.BuiltIn.string().(\"hi\")\n    {:ok, \"hi\"}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.string().(~c\"hi\")\n    ...> Error.reason(e)\n    :not_a_string\n\n    iex> {:error, e} = Data.Parser.BuiltIn.string().(:hi)\n    ...> Error.reason(e)\n    :not_a_string","ref":"Data.Parser.BuiltIn.html#string/0-examples"},{"type":"function","title":"Data.Parser.BuiltIn.string_of/1","doc":"Creates a parser that successfully parses strings representing either\n`Integer`s or `Float`s.\n\nReturns a domain error detailing the parse failure on bad inputs.\n\nLook out! Partial results, such as that of Integer.parse(\"abc123\"), still\ncount as errors!","ref":"Data.Parser.BuiltIn.html#string_of/1"},{"type":"function","title":"Examples - Data.Parser.BuiltIn.string_of/1","doc":"iex> Data.Parser.BuiltIn.string_of(Float).(\"1.1\")\n    {:ok, 1.1}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.string_of(Float).(\"abc\")\n    ...> Error.reason(e)\n    :not_parseable_as_float\n    ...> Error.details(e)\n    %{input: \"abc\", native_parser_output: :error}\n\n    iex> Data.Parser.BuiltIn.string_of(Integer).(\"1234567890\")\n    {:ok, 1234567890}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.string_of(Integer).(\"123abc\")\n    ...> Error.reason(e)\n    :not_parseable_as_integer\n    ...> Error.details(e)\n    %{input: \"123abc\", native_parser_output: {123, \"abc\"}}\n\n    iex> {:error, e} = Data.Parser.BuiltIn.string_of(Integer).([])\n    ...> Error.reason(e)\n    :not_a_string","ref":"Data.Parser.BuiltIn.html#string_of/1-examples"},{"type":"module","title":"Data.Parser.KV","doc":"Creates parsers that accept KeyValue-style `Enum`s as input.\n\nIn particular, KV parsers work with:\n\n- maps (e.g. `%{\"hello\" => \"world\"}`)\n- `Keyword.t`s (e.g. `[hello: \"world\"]`)\n- Lists of pairs (e.g. `[{\"hello\", \"world\"}]`)\n\nKV parsers are higher-order parsers, and operate in roughly the same way as\n`Data.Parser.list/1` or `Data.Parser.set/1`, but their definition is slightly\nmore involved. A KV parser is created with a list of `field_spec`s, where\neach `field_spec` defines what fields of the input to look at, and what\nparsers to run on them.\n\nHere are some examples of `field_spec`s and their parsing behavior:\n\n###  `{:username, Data.Parser.BuiltIn.string()}`\n\nThis spec says that the input must contain a `:username` field, and the value\nof that field must satisfy `Data.Parser.BuiltIn.string/0`.  The output map\nwill contain the key-value pair `username: \"some string\"`.\n\nIf the field cannot be parsed successfully, the entire KV parser will return\n`{:error, domain_error_with_details_on_parse_failure}`.\n\nIf the field is not present, the entire KV parser will return\n`{:error, domain_error_with_details_about_field_not_found}`\n\n\n###  `{:birthday, Data.Parser.BuiltIn.date(), optional: true}`\n\nThis spec says that the input *may* contain a `:birthday` field. If the field\ndoes exist, it must satisfy `Data.Parser.BuiltIn.date/0`.\n\nIf the field exists and parses successfully, the output map will contain the\nkey-value pair `birthday: {:just, ~D[1983-07-18]}`.\n\nIf the field does not exist, the output map will contain the key-value pair\n`birthday: :nothing`.\n\nIf the field cannot be parsed successfully, the entire KV parser will return\n`{:error, domain_error_with_parse_failure_details}`.\n\n\n### `{:country, MyApp.country_parser(), default: \"Canada\"}`\n\n\nThis spec says that the input *may* contain a `:country` field, and if so, the\nvalue of that field must parse successfully with `MyApp.country_parser/0`.\n\nIf the field exists and parses successfully, the output map will contain a\nkey-value pair such as: `country: \"Indonesia\"`.\n\nIf the field cannot be parsed successfully, the entire Constructor will return\n`{:error, domain_error_with_details_on_parse_failure}`.\n\nIf the field does *not* exist, the `default` value will be used. In this\ncase, the output will contain the key-value pair: `country: \"Canada\"`\n\n\n### `{:country, MyApp.country_parser(), nullable: true}`\n\n\nThis spec says that the input *must* contain a `:country` field, and if so,\nthe value of that field must parse successfully with `MyApp.country_parser/0`\nOR be equal to `nil`.\n\nIf the field exists and parses successfully, the output map will contain a\nkey-value pair such as: `country: \"Indonesia\"`.\n\nIf the field cannot be parsed successfully, the entire Constructor will return\n`{:error, domain_error_with_details_on_parse_failure}`. However, if the value\nof the field is `nil`, this is treated as a successful parse.\n\nIf the field does *not* exist, the parser will fail.\n\nNote that a field spec specified as `nullable: true` cannot also contain\neither the `optional: true` or `default: x` options.\nThis is an illegal spec and will not be constructed, resulting in\nan `{:error, domain_error}` tuple, with `:invalid_field_spec` as the\nerror reason.\n\n\n### `{:country, MyApp.country_parser(), from: :countryName}`\n\n\nThis spec says that the parser will use the data from `:countryName` in the\ninput map. If the value under this key satisfies the\n`MyApp.country_parser()`, then the resulting value will be placed under the\n`:country` field.\n\nNote that the `from` keyname MUST always be specified as an atom, but it will\nbe applied automatically to string keys. If the input contains *both* a\nstring key and an atom key, the atom key will take priority.\n\n\n### `{:point, MyApp.point_parser(), recurse: true}`\n\nSometimes you want to run several different parsers on the same input\nmap. For example, let's say your input looks like this:\n\n```\n%{x: 12,\n  y: -10,\n  value: 34,\n  name: \"exploding_barrel\"}\n```\n\nBut the data structure you want after parsing looks like this:\n\n```\n%{point: %{x: 12, y: -10},\n  value: 34,\n  name: \"exploding_barrel\"}\n```\n\nAnd you have MyApp.point_parser() which accepts a map with `:x` and `:y`\ninteger keys and constructs `%{x: integer(), y: integer()}`.\n\nYou can define a field_spec with `recurse: true` and have that particular\nparser get run on its *parent input map*, not on the value of a field.","ref":"Data.Parser.KV.html"},{"type":"opaque","title":"Data.Parser.KV.field/2","doc":"A structure representing a `Data.Parser.t(a,b)` lifted to operate on a KV.","ref":"Data.Parser.KV.html#t:field/2"},{"type":"type","title":"Data.Parser.KV.field_name/0","doc":"KV parsers accept `atom()`s as key names, but will work on inputs where\nthe keys are `String.t()`s as well.","ref":"Data.Parser.KV.html#t:field_name/0"},{"type":"type","title":"Data.Parser.KV.field_opts/1","doc":"Options to relax requirements on the fields.\n\nThis is a list that consists of zero or one of the below options:\n`{:optional, bool()}`\n`{:default, any}`\n`{:from, field_name()}`","ref":"Data.Parser.KV.html#t:field_opts/1"},{"type":"type","title":"Data.Parser.KV.field_spec/2","doc":"A 2-tuple or 3-tuple describing the field to parse and parsing semantics.\n\n`{field_name, parser}`\n`{field_name, parser, opts}`","ref":"Data.Parser.KV.html#t:field_spec/2"},{"type":"type","title":"Data.Parser.KV.input/0","doc":"KV parsers accept either a map or a `Keyword.t` as input.","ref":"Data.Parser.KV.html#t:input/0"},{"type":"function","title":"Data.Parser.KV.new/1","doc":"Given a list of `field_spec`s, verify that all specs are well-formed and\nreturn `{:ok, parser}`, where `parser` will accept a `map` or `Keyword` input\nand apply the appropriate parsers to their corresponding fields.\n\nIf the `field_spec`s are not well-formed, return `{:error, Error.t}` with details\nabout the invalid `field_spec`s.","ref":"Data.Parser.KV.html#new/1"},{"type":"function","title":"Examples - Data.Parser.KV.new/1","doc":"iex> {:ok, p} = Data.Parser.KV.new([{:username, Data.Parser.BuiltIn.string()}])\n    ...> p.(username: \"johndoe\")\n    {:ok, %{username: \"johndoe\"}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:username, Data.Parser.BuiltIn.string()}])\n    ...> p.(%{\"username\" => \"johndoe\"})\n    {:ok, %{username: \"johndoe\"}}\n\n    iex> {:error, e} = Data.Parser.KV.new([\"not a spec\"])\n    ...> e.reason\n    :invalid_field_spec\n    ...> e.details\n    %{spec: \"not a spec\"}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), optional: true}])\n    ...> p.(a: 1)\n    {:ok, %{a: {:just, 1}}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), optional: true}])\n    ...> p.([])\n    {:ok, %{a: :nothing}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), nullable: true}])\n    ...> {:error, e} = p.([])\n    ...> Error.reason(e)\n    :field_not_found_in_input\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), nullable: true}])\n    ...> p.([a: nil])\n    {:ok, %{a: nil}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), nullable: true}])\n    ...> p.([a: 1])\n    {:ok, %{a: 1}}\n\n    iex> {:error, e} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), nullable: true, default: nil}])\n    ...> Error.reason(e)\n    :invalid_field_spec\n\n    iex> {:error, e} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), nullable: true, optional: true}])\n    ...> Error.reason(e)\n    :invalid_field_spec\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:b, Data.Parser.BuiltIn.integer(), default: 0}])\n    ...> p.([])\n    {:ok, %{b: 0}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:b, Data.Parser.BuiltIn.integer(), default: 0}])\n    ...> p.(b: 10)\n    {:ok, %{b: 10}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:b, Data.Parser.BuiltIn.integer(), default: 0}])\n    ...> {:error, e} = p.(b: \"i am of the wrong type\")\n    ...> Error.reason(e)\n    :failed_to_parse_field\n    ...> {:just, inner_error} = Error.caused_by(e)\n    ...> Error.reason(inner_error)\n    :not_an_integer\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), from: :theAValue}])\n    ...> p.(%{theAValue: 123})\n    {:ok, %{a: 123}}\n\n    iex> {:ok, p} = Data.Parser.KV.new([{:a, Data.Parser.BuiltIn.integer(), from: :aStringKey}])\n    ...> p.(%{\"aStringKey\" => 1234})\n    {:ok, %{a: 1234}}\n\n    iex> {:ok, point} = Data.Parser.KV.new([{:x, Data.Parser.BuiltIn.integer()}, {:y, Data.Parser.BuiltIn.integer()}])\n    ...> {:ok, item} = Data.Parser.KV.new([{:point, point, recurse: true}, {:value, Data.Parser.BuiltIn.integer()}])\n    ...> item.(%{x: 1, y: -1, value: 34})\n    {:ok, %{value: 34, point: %{x: 1, y: -1}}}\n\n    iex> {:ok, point} = Data.Parser.KV.new([{:x, Data.Parser.BuiltIn.integer()}, {:y, Data.Parser.BuiltIn.integer()}])\n    ...> {:ok, item} = Data.Parser.KV.new([{:point, point, recurse: true}, {:value, Data.Parser.BuiltIn.integer()}])\n    ...> {:error, e} = item.(%{x: \"wrong\", y: -1, value: 34})\n    ...> {:just, e2} = e.caused_by\n    ...> e2.reason\n    :failed_to_parse_field\n\n\n    iex> {:ok, point} = Data.Parser.KV.new([{:x, Data.Parser.BuiltIn.integer()}, {:y, Data.Parser.BuiltIn.integer()}])\n    ...> {:ok, item} = Data.Parser.KV.new([{:point, point, recurse: true}, {:value, Data.Parser.BuiltIn.integer()}])\n    ...> {:error, e} = item.(%{y: -1, value: 34})\n    ...> {:just, e2} = e.caused_by\n    ...> e2.reason\n    :field_not_found_in_input","ref":"Data.Parser.KV.html#new/1-examples"},{"type":"function","title":"Data.Parser.KV.one/1","doc":"Given one `field_spec`, verify that it is well-formed and\nreturn `{:ok, parser}`, where `parser` will accept a `map` or `Keyword` input\nand attempt to parse that single field out of the map.\n\nAny `field_spec` with default semantics will be 'lifted' to also accept the\ndefault value if present under the key.\n\nAny `field_spec` with optional semantics will be stripped of these, so that\nit can 'select' only fields which exist.\n\nIf the `field_spec` is not well-formed, return `{:error, Error.t}` with details\nabout the invalid `field_spec`.","ref":"Data.Parser.KV.html#one/1"},{"type":"function","title":"Examples - Data.Parser.KV.one/1","doc":"iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string()})\n    ...> p.(username: \"johndoe\")\n    {:ok, %{username: \"johndoe\"}}\n\n    iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string()})\n    ...> p.(%{\"username\" => \"johndoe\"})\n    {:ok, %{username: \"johndoe\"}}\n\n    iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string(), default: nil})\n    ...> p.(username: nil)\n    {:ok, %{username: nil}}\n\n    iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string(), default: nil})\n    ...> {:error, e} = p.(%{username: [1,2,3]})\n    ...> Error.reason(e)\n    :failed_to_parse_field\n\n    iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string(), default: nil})\n    ...> {:error, e} = p.(%{\"a\" => \"b\"})\n    ...> Error.reason(e)\n    :field_not_found_in_input\n\n    iex> {:ok, p} = Data.Parser.KV.one({:username, Data.Parser.BuiltIn.string(), optiona: true})\n    ...> {:error, e} = p.(%{\"a\" => \"b\"})\n    ...> Error.reason(e)\n    :field_not_found_in_input","ref":"Data.Parser.KV.html#one/1-examples"},{"type":"extras","title":"Data","doc":"# Data [![CircleCI](https://circleci.com/gh/well-ironed/data.svg?style=svg)](https://circleci.com/gh/well-ironed/data)\n\nExtensions to Elixir data structures.\n\nCurrently, this library contains 3 major components:\n\n - `Data.Enum` — extensions to `Enum`s.\n - `Data.Parser` — parsers and parser combinators for BEAM data types.\n - `Data.Constructor` — smart constructors (struct building w/ validation)\n   based on `Data.Parser`s.\n\n\nFor detailed documentation, see [hex.pm](https://hexdocs.pm/data/readme.html).","ref":"readme.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}